@@ -784,11 +784,11 @@ class Languages < Hash
 	"exceptions" => false,
 	"message" => "Spanish hyphenation patterns",
 
-	"version"       => "4.6",
-	"last_modified" => "2010-05-18",
+	"version"       => "4.7",
+	"last_modified" => "2015-11-12",
 	"type"          => "dictionary",
 	"authors"       => [ "javier_bezos" ],
-	"licence"       => "LPPL",
+	"licence"       => "MIT/X11",
 	"description_s" => "Spanish hyphenation patterns",
 	"description_l" => [
 		#......................................................................#
@@ -886,7 +886,7 @@ class Languages < Hash
 	"last_modified" => "2004-04-13",
 	"type"          => "dictionary",
 	"authors"       => [ "een_saar" ],
-	"licence"       => "LPPL",
+	"licence"       => "LPPL|MIT", # Dual MIT-LPPL
 	"description_s" => "Estonian hyphenation patterns",
 	"description_l" => [
 		#......................................................................#
@@ -982,7 +982,7 @@ class Languages < Hash
 	"last_modified" => "2010-05",
 	"type"          => "rules",
 	"authors"       => ["sahak_petrosyan"],
-	"licence"       => "LPGL",
+	"licence"       => "LGPL",
 	"description_s" => "Armenian hyphenation patterns",
 	"description_l" => [
 		#......................................................................#
@@ -1344,6 +1344,32 @@ class Languages < Hash
 		"to their meaning."
 	],
 },
+# occitan
+{
+	"code" => "oc",
+	"name" => "occitan",
+	"use_new_loader" => true,
+	"use_old_patterns" => false,
+	"hyphenmin" => [2,2],
+	"encoding" => "ec",
+	"exceptions" => false,
+	"message" => "Occitan hyphenation patterns",
+
+	"version"       => "0.1",
+	"last_modified" => "2016-02-04",
+	"type"          => "rules",
+	"authors"       => [ "claudio_beccari" ],
+	"licence"       => "LPPL",
+	"description_s" => "Occitan hyphenation patterns",
+	"description_l" => [
+		#......................................................................#
+		"Hyphenation patterns for Occitan in T1/EC and UTF-8 encodings.",
+		"They are supposed to be valid for all the Occitan variants",
+		"spoken and written in the wide area called 'Occitanie' by the French.",
+		"It ranges from the Val d'Aran within Catalunya, to the South Western",
+		"Italian Alps encompassing the southern half of the French pentagon.",
+	],
+},
 # polish
 {
 	"code" => "pl",
@@ -1386,7 +1412,7 @@ class Languages < Hash
 	"last_modified" => "1996-07-21",
 	"type"          => "rules", # TODO: we could create a generating script
 	"authors"       => ["pedro_j_de_rezende", "j_joao_dias_almeida"],
-	"licence"       => "LPPL",
+	"licence"       => "BSD-3",
 	"description_s" => "Portuguese hyphenation patterns",
 	"description_l" => [
 		#......................................................................#
@@ -1832,11 +1858,11 @@ class Languages < Hash
 	"message" => "Assamese hyphenation patterns",
 
 	# this is true for all Indic patterns
-	"version"       => "0.5.3",
-	"last_modified" => "2010-05-01",
+	"version"       => "0.9.0",
+	"last_modified" => "2016-01-16",
 	"type"          => "rules",
 	"authors"       => ["santhosh_thottingal"],
-	"licence"       => "LGPL",
+	"licence"       => "MIT",
 	"description_s" => "Indic hyphenation patterns",
 	"description_l" => [
 		#......................................................................#
@@ -1967,7 +1993,7 @@ class Languages < Hash
 	"message" => "Thai hyphenation patterns",
 
 	"version"       => nil,
-	"last_modified" => "2014-08-22",
+	"last_modified" => "2015-05-07",
 	"type"          => "dictionary",
 	"authors"       => [ "theppitak_karoonboonyanan" ],
 	"licence"       => "LPPL",
