@@ -1,6 +1,6 @@
 -- merged file : lualibs-extended-merged.lua
 -- parent file : lualibs-extended.lua
--- merge date  : Mon May  5 07:37:22 2014
+-- merge date  : Wed Dec  2 07:52:44 2015
 
 do -- begin closure to overcome local limits and interference
 
@@ -37,6 +37,11 @@ else
 end
 if not number then number={} end 
 local stripper=patterns.stripzeros
+local newline=patterns.newline
+local endofstring=patterns.endofstring
+local whitespace=patterns.whitespace
+local spacer=patterns.spacer
+local spaceortab=patterns.spaceortab
 local function points(n)
   n=tonumber(n)
   return (not n or n==0) and "0pt" or lpegmatch(stripper,format("%.5fpt",n/65536))
@@ -47,12 +52,12 @@ local function basepoints(n)
 end
 number.points=points
 number.basepoints=basepoints
-local rubish=patterns.spaceortab^0*patterns.newline
-local anyrubish=patterns.spaceortab+patterns.newline
+local rubish=spaceortab^0*newline
+local anyrubish=spaceortab+newline
 local anything=patterns.anything
-local stripped=(patterns.spaceortab^1/"")*patterns.newline
+local stripped=(spaceortab^1/"")*newline
 local leading=rubish^0/""
-local trailing=(anyrubish^1*patterns.endofstring)/""
+local trailing=(anyrubish^1*endofstring)/""
 local redundant=rubish^3/"\n"
 local pattern=Cs(leading*(trailing+redundant+stripped+anything)^0)
 function strings.collapsecrlf(str)
@@ -98,17 +103,13 @@ local pattern=Carg(1)/function(t)
      else
        return ""
      end
-   end+patterns.newline*Cp()/function(position)
+   end+newline*Cp()/function(position)
      extra,start=0,position
    end+patterns.anything
  )^1)
 function strings.tabtospace(str,tab)
   return lpegmatch(pattern,str,1,tab or 7)
 end
-local newline=patterns.newline
-local endofstring=patterns.endofstring
-local whitespace=patterns.whitespace
-local spacer=patterns.spacer
 local space=spacer^0
 local nospace=space/""
 local endofline=nospace*newline
@@ -132,10 +133,12 @@ local striplinepatterns={
   ["retain"]=p_retain_normal,
   ["retain and collapse"]=p_retain_collapse,
   ["retain and no empty"]=p_retain_noempty,
+  ["collapse"]=patterns.collapser,
 }
+setmetatable(striplinepatterns,{ __index=function(t,k) return p_prune_collapse end })
 strings.striplinepatterns=striplinepatterns
 function strings.striplines(str,how)
-  return str and lpegmatch(how and striplinepatterns[how] or p_prune_collapse,str) or str
+  return str and lpegmatch(striplinepatterns[how],str) or str
 end
 strings.striplong=strings.striplines
 function strings.nice(str)
@@ -170,15 +173,21 @@ function string.autosingle(s,sep)
   end
   return ("'"..tostring(s).."'")
 end
-local tracedchars={}
+local tracedchars={ [0]=
+  "[null]","[soh]","[stx]","[etx]","[eot]","[enq]","[ack]","[bel]",
+  "[bs]","[ht]","[lf]","[vt]","[ff]","[cr]","[so]","[si]",
+  "[dle]","[dc1]","[dc2]","[dc3]","[dc4]","[nak]","[syn]","[etb]",
+  "[can]","[em]","[sub]","[esc]","[fs]","[gs]","[rs]","[us]",
+  "[space]",
+}
 string.tracedchars=tracedchars
 strings.tracers=tracedchars
 function string.tracedchar(b)
   if type(b)=="number" then
-    return tracedchars[b] or (utfchar(b).." (U+"..format('%05X',b)..")")
+    return tracedchars[b] or (utfchar(b).." (U+"..format("%05X",b)..")")
   else
     local c=utfbyte(b)
-    return tracedchars[c] or (b.." (U+"..format('%05X',c)..")")
+    return tracedchars[c] or (b.." (U+"..(c and format("%05X",c) or "?????")..")")
   end
 end
 function number.signed(i)
@@ -315,9 +324,13 @@ local format_f=function(f)
   n=n+1
   return format("format('%%%sf',a%s)",f,n)
 end
-local format_F=function(f)
+local format_F=function(f) 
   n=n+1
-  return format("((a%s == 0 and '0') or (a%s == 1 and '1') or format('%%%sf',a%s))",n,n,f,n)
+  if not f or f=="" then
+    return format("(((a%s > -0.0000000005 and a%s < 0.0000000005) and '0') or format((a%s %% 1 == 0) and '%%i' or '%%.9f',a%s))",n,n,n,n)
+  else
+    return format("format((a%s %% 1 == 0) and '%%i' or '%%%sf',a%s)",n,f,n)
+  end
 end
 local format_g=function(f)
   n=n+1
@@ -544,7 +557,6 @@ local builder=Cs { "start",
 +V("j")+V("J") 
 +V("m")+V("M") 
 +V("z")
-+V("*") 
       )+V("*")
     )*(P(-1)+Carg(1))
   )^0,
@@ -588,6 +600,7 @@ local builder=Cs { "start",
   ["a"]=(prefix_any*P("a"))/format_a,
   ["A"]=(prefix_any*P("A"))/format_A,
   ["*"]=Cs(((1-P("%"))^1+P("%%")/"%%")^1)/format_rest,
+  ["?"]=Cs(((1-P("%"))^1        )^1)/format_rest,
   ["!"]=Carg(2)*prefix_any*P("!")*C((1-P("!"))^1)*P("!")/format_extension,
 }
 local direct=Cs (
@@ -662,6 +675,19 @@ else
   add(formatters,"tex",[[lpegmatch(texescape,%s)]],{ texescape=lpeg.patterns.texescape })
   add(formatters,"lua",[[lpegmatch(luaescape,%s)]],{ luaescape=lpeg.patterns.luaescape })
 end
+local dquote=patterns.dquote 
+local equote=patterns.escaped+dquote/'\\"'+1
+local space=patterns.space
+local cquote=Cc('"')
+local pattern=Cs(dquote*(equote-P(-2))^0*dquote)          
++Cs(cquote*(equote-space)^0*space*equote^0*cquote) 
+function string.optionalquoted(str)
+  return lpegmatch(pattern,str) or str
+end
+local pattern=Cs((newline/(os.newline or "\r")+1)^0)
+function string.replacenewlines(str)
+  return lpegmatch(pattern,str)
+end
 
 end -- closure
 
@@ -678,7 +704,7 @@ utilities=utilities or {}
 utilities.tables=utilities.tables or {}
 local tables=utilities.tables
 local format,gmatch,gsub,sub=string.format,string.gmatch,string.gsub,string.sub
-local concat,insert,remove=table.concat,table.insert,table.remove
+local concat,insert,remove,sort=table.concat,table.insert,table.remove,table.sort
 local setmetatable,getmetatable,tonumber,tostring=setmetatable,getmetatable,tonumber,tostring
 local type,next,rawset,tonumber,tostring,load,select=type,next,rawset,tonumber,tostring,load,select
 local lpegmatch,P,Cs,Cc=lpeg.match,lpeg.P,lpeg.Cs,lpeg.Cc
@@ -755,6 +781,15 @@ function tables.removevalue(t,value)
     end
   end
 end
+function tables.replacevalue(t,oldvalue,newvalue)
+  if oldvalue and newvalue then
+    for i=1,#t do
+      if t[i]==oldvalue then
+        t[i]=newvalue
+      end
+    end
+  end
+end
 function tables.insertbeforevalue(t,value,extra)
   for i=1,#t do
     if t[i]==extra then
@@ -1085,21 +1120,24 @@ local f_val_str=formatters["%w%q,"]
 local f_val_boo=formatters["%w%l,"]
 local f_val_not=formatters["%w{},"]
 local f_val_seq=formatters["%w{ %, t },"]
+local f_fin_seq=formatters[" %, t }"]
 local f_table_return=formatters["return {"]
 local f_table_name=formatters["%s={"]
 local f_table_direct=formatters["{"]
 local f_table_entry=formatters["[%q]={"]
 local f_table_finish=formatters["}"]
 local spaces=utilities.strings.newrepeater(" ")
-local serialize=table.serialize
-function table.serialize(root,name,specification)
+local original_serialize=table.serialize
+local function serialize(root,name,specification)
   if type(specification)=="table" then
-    return serialize(root,name,specification) 
+    return original_serialize(root,name,specification) 
   end
-  local t 
+  local t  
   local n=1
+  local unknown=false
   local function simple_table(t)
-    if #t>0 then
+    local nt=#t
+    if nt>0 then
       local n=0
       for _,v in next,t do
         n=n+1
@@ -1107,24 +1145,40 @@ function table.serialize(root,name,specification)
           return nil
         end
       end
-      if n==#t then
+      local haszero=t[0]
+      if n==nt then
         local tt={}
-        local nt=0
-        for i=1,#t do
+        for i=1,nt do
           local v=t[i]
           local tv=type(v)
-          nt=nt+1
           if tv=="number" then
-            tt[nt]=v
+            tt[i]=v 
           elseif tv=="string" then
-            tt[nt]=format("%q",v) 
+            tt[i]=format("%q",v) 
           elseif tv=="boolean" then
-            tt[nt]=v and "true" or "false"
+            tt[i]=v and "true" or "false"
           else
             return nil
           end
         end
         return tt
+      elseif haszero and (n==nt+1) then
+        local tt={}
+        for i=0,nt do
+          local v=t[i]
+          local tv=type(v)
+          if tv=="number" then
+            tt[i+1]=v 
+          elseif tv=="string" then
+            tt[i+1]=format("%q",v) 
+          elseif tv=="boolean" then
+            tt[i+1]=v and "true" or "false"
+          else
+            return nil
+          end
+        end
+        tt[1]="[0] = "..tt[1]
+        return tt
       end
     end
     return nil
@@ -1148,7 +1202,7 @@ function table.serialize(root,name,specification)
       end
       depth=depth+1
     end
-    if root and next(root) then
+    if root and next(root)~=nil then
       local first=nil
       local last=0
       last=#root
@@ -1167,13 +1221,13 @@ function table.serialize(root,name,specification)
         local v=root[k]
         local tv=type(v)
         local tk=type(k)
-        if first and tk=="number" and k>=first and k<=last then
+        if first and tk=="number" and k<=last and k>=first then
           if tv=="number" then
             n=n+1 t[n]=f_val_num(depth,v)
           elseif tv=="string" then
             n=n+1 t[n]=f_val_str(depth,v)
           elseif tv=="table" then
-            if not next(v) then
+            if next(v)==nil then 
               n=n+1 t[n]=f_val_not(depth)
             else
               local st=simple_table(v)
@@ -1185,6 +1239,8 @@ function table.serialize(root,name,specification)
             end
           elseif tv=="boolean" then
             n=n+1 t[n]=f_val_boo(depth,v)
+          elseif unknown then
+            n=n+1 t[n]=f_val_str(depth,tostring(v))
           end
         elseif tv=="number" then
           if tk=="number" then
@@ -1193,6 +1249,8 @@ function table.serialize(root,name,specification)
             n=n+1 t[n]=f_key_str_value_num(depth,k,v)
           elseif tk=="boolean" then
             n=n+1 t[n]=f_key_boo_value_num(depth,k,v)
+          elseif unknown then
+            n=n+1 t[n]=f_key_str_value_num(depth,tostring(k),v)
           end
         elseif tv=="string" then
           if tk=="number" then
@@ -1201,15 +1259,19 @@ function table.serialize(root,name,specification)
             n=n+1 t[n]=f_key_str_value_str(depth,k,v)
           elseif tk=="boolean" then
             n=n+1 t[n]=f_key_boo_value_str(depth,k,v)
+          elseif unknown then
+            n=n+1 t[n]=f_key_str_value_str(depth,tostring(k),v)
           end
         elseif tv=="table" then
-          if not next(v) then
+          if next(v)==nil then
             if tk=="number" then
-              n=n+1 t[n]=f_key_num_value_not(depth,k,v)
+              n=n+1 t[n]=f_key_num_value_not(depth,k)
             elseif tk=="string" then
-              n=n+1 t[n]=f_key_str_value_not(depth,k,v)
+              n=n+1 t[n]=f_key_str_value_not(depth,k)
             elseif tk=="boolean" then
-              n=n+1 t[n]=f_key_boo_value_not(depth,k,v)
+              n=n+1 t[n]=f_key_boo_value_not(depth,k)
+            elseif unknown then
+              n=n+1 t[n]=f_key_str_value_not(depth,tostring(k))
             end
           else
             local st=simple_table(v)
@@ -1221,6 +1283,8 @@ function table.serialize(root,name,specification)
               n=n+1 t[n]=f_key_str_value_seq(depth,k,st)
             elseif tk=="boolean" then
               n=n+1 t[n]=f_key_boo_value_seq(depth,k,st)
+            elseif unknown then
+              n=n+1 t[n]=f_key_str_value_seq(depth,tostring(k),st)
             end
           end
         elseif tv=="boolean" then
@@ -1230,6 +1294,18 @@ function table.serialize(root,name,specification)
             n=n+1 t[n]=f_key_str_value_boo(depth,k,v)
           elseif tk=="boolean" then
             n=n+1 t[n]=f_key_boo_value_boo(depth,k,v)
+          elseif unknown then
+            n=n+1 t[n]=f_key_str_value_boo(depth,tostring(k),v)
+          end
+        else
+          if tk=="number" then
+            n=n+1 t[n]=f_key_num_value_str(depth,k,tostring(v))
+          elseif tk=="string" then
+            n=n+1 t[n]=f_key_str_value_str(depth,k,tostring(v))
+          elseif tk=="boolean" then
+            n=n+1 t[n]=f_key_boo_value_str(depth,k,tostring(v))
+          elseif unknown then
+            n=n+1 t[n]=f_key_str_value_str(depth,tostring(k),tostring(v))
           end
         end
       end
@@ -1261,14 +1337,23 @@ function table.serialize(root,name,specification)
       local dummy=root._w_h_a_t_e_v_e_r_
       root._w_h_a_t_e_v_e_r_=nil
     end
-    if next(root) then
-      do_serialize(root,name,1,0)
+    if next(root)~=nil then
+      local st=simple_table(root)
+      if st then
+        return t[1]..f_fin_seq(st) 
+      else
+        do_serialize(root,name,1,0)
+      end
     end
   end
   n=n+1
   t[n]=f_table_finish()
   return concat(t,"\n")
 end
+table.serialize=serialize
+if setinspector then
+  setinspector("table",function(v) if type(v)=="table" then print(serialize(v,"table")) return true end end)
+end
 
 end -- closure
 
@@ -1350,39 +1435,32 @@ local f_index={
   ["table"]=f_table,
   ["number"]=f_number,
 }
-local t_index={
-  ["empty"]={ __index=f_empty },
-  ["self"]={ __index=f_self  },
-  ["table"]={ __index=f_table },
-  ["number"]={ __index=f_number },
-}
 function table.setmetatableindex(t,f)
   if type(t)~="table" then
     f,t=t,{}
   end
   local m=getmetatable(t)
+  local i=f_index[f] or f
   if m then
-    m.__index=f_index[f] or f
+    m.__index=i
   else
-    setmetatable(t,t_index[f] or { __index=f })
+    setmetatable(t,{ __index=i })
   end
   return t
 end
 local f_index={
   ["ignore"]=f_ignore,
 }
-local t_index={
-  ["ignore"]={ __newindex=f_ignore },
-}
 function table.setmetatablenewindex(t,f)
   if type(t)~="table" then
     f,t=t,{}
   end
   local m=getmetatable(t)
+  local i=f_index[f] or f
   if m then
-    m.__newindex=f_index[f] or f
+    m.__newindex=i
   else
-    setmetatable(t,t_index[f] or { __newindex=f })
+    setmetatable(t,{ __newindex=i })
   end
   return t
 end
@@ -1436,6 +1514,8 @@ local patterns=parsers.patterns or {}
 parsers.patterns=patterns
 local setmetatableindex=table.setmetatableindex
 local sortedhash=table.sortedhash
+local sortedkeys=table.sortedkeys
+local tohash=table.tohash
 local digit=R("09")
 local space=P(' ')
 local equal=P("=")
@@ -1485,9 +1565,7 @@ patterns.settings_to_hash_a=pattern_a_s
 patterns.settings_to_hash_b=pattern_b_s
 patterns.settings_to_hash_c=pattern_c_s
 function parsers.make_settings_to_hash_pattern(set,how)
-  if type(str)=="table" then
-    return set
-  elseif how=="strict" then
+  if how=="strict" then
     return (pattern_c/set)^1
   elseif how=="tolerant" then
     return (pattern_b/set)^1
@@ -1496,7 +1574,9 @@ function parsers.make_settings_to_hash_pattern(set,how)
   end
 end
 function parsers.settings_to_hash(str,existing)
-  if type(str)=="table" then
+  if not str or str=="" then
+    return {}
+  elseif type(str)=="table" then
     if existing then
       for k,v in next,str do
         existing[k]=v
@@ -1505,16 +1585,16 @@ function parsers.settings_to_hash(str,existing)
     else
       return str
     end
-  elseif str and str~="" then
+  else
     hash=existing or {}
     lpegmatch(pattern_a_s,str)
     return hash
-  else
-    return {}
   end
 end
 function parsers.settings_to_hash_tolerant(str,existing)
-  if type(str)=="table" then
+  if not str or str=="" then
+    return {}
+  elseif type(str)=="table" then
     if existing then
       for k,v in next,str do
         existing[k]=v
@@ -1523,16 +1603,16 @@ function parsers.settings_to_hash_tolerant(str,existing)
     else
       return str
     end
-  elseif str and str~="" then
+  else
     hash=existing or {}
     lpegmatch(pattern_b_s,str)
     return hash
-  else
-    return {}
   end
 end
 function parsers.settings_to_hash_strict(str,existing)
-  if type(str)=="table" then
+  if not str or str=="" then
+    return nil
+  elseif type(str)=="table" then
     if existing then
       for k,v in next,str do
         existing[k]=v
@@ -1545,8 +1625,6 @@ function parsers.settings_to_hash_strict(str,existing)
     hash=existing or {}
     lpegmatch(pattern_c_s,str)
     return next(hash) and hash
-  else
-    return nil
   end
 end
 local separator=comma*space^0
@@ -1554,10 +1632,10 @@ local value=P(lbrace*C((nobrace+nestedbraces)^0)*rbrace)+C((nestedbraces+(1-comm
 local pattern=spaces*Ct(value*(separator*value)^0)
 patterns.settings_to_array=pattern
 function parsers.settings_to_array(str,strict)
-  if type(str)=="table" then
-    return str
-  elseif not str or str=="" then
+  if not str or str=="" then
     return {}
+  elseif type(str)=="table" then
+    return str
   elseif strict then
     if find(str,"{",1,true) then
       return lpegmatch(pattern,str)
@@ -1570,11 +1648,30 @@ function parsers.settings_to_array(str,strict)
     return { str }
   end
 end
-local separator=space^0*comma*space^0
-local value=P(lbrace*C((nobrace+nestedbraces)^0)*rbrace)+C((nestedbraces+(1-(space^0*(comma+P(-1)))))^0)
-local withvalue=Carg(1)*value/function(f,s) return f(s) end
-local pattern_a=spaces*Ct(value*(separator*value)^0)
-local pattern_b=spaces*withvalue*(separator*withvalue)^0
+local cache_a={}
+local cache_b={}
+function parsers.groupedsplitat(symbol,withaction)
+  if not symbol then
+    symbol=","
+  end
+  local pattern=(withaction and cache_b or cache_a)[symbol]
+  if not pattern then
+    local symbols=S(symbol)
+    local separator=space^0*symbols*space^0
+    local value=P(lbrace*C((nobrace+nestedbraces)^0)*rbrace)+C((nestedbraces+(1-(space^0*(symbols+P(-1)))))^0)
+    if withaction then
+      local withvalue=Carg(1)*value/function(f,s) return f(s) end
+      pattern=spaces*withvalue*(separator*withvalue)^0
+      cache_b[symbol]=pattern
+    else
+      pattern=spaces*Ct(value*(separator*value)^0)
+      cache_a[symbol]=pattern
+    end
+  end
+  return pattern
+end
+local pattern_a=parsers.groupedsplitat(",",false)
+local pattern_b=parsers.groupedsplitat(",",true)
 function parsers.stripped_settings_to_array(str)
   if not str or str=="" then
     return {}
@@ -1599,8 +1696,8 @@ function parsers.add_settings_to_array(t,str)
 end
 function parsers.hash_to_string(h,separator,yes,no,strict,omit)
   if h then
-    local t,tn,s={},0,table.sortedkeys(h)
-    omit=omit and table.tohash(omit)
+    local t,tn,s={},0,sortedkeys(h)
+    omit=omit and tohash(omit)
     for i=1,#s do
       local key=s[i]
       if not omit or not omit[key] then
@@ -1636,12 +1733,9 @@ function parsers.array_to_string(a,separator)
     return ""
   end
 end
-function parsers.settings_to_set(str,t) 
-  t=t or {}
-  for s in gmatch(str,"[^, ]+") do 
-    t[s]=true
-  end
-  return t
+local pattern=Cf(Ct("")*Cg(C((1-S(", "))^1)*S(", ")^0*Cc(true))^1,rawset)
+function utilities.parsers.settings_to_set(str,t)
+  return str and lpegmatch(pattern,str) or {}
 end
 function parsers.simple_hash_to_string(h,separator)
   local t,tn={},0
@@ -1653,12 +1747,16 @@ function parsers.simple_hash_to_string(h,separator)
   end
   return concat(t,separator or ",")
 end
-local str=C((1-whitespace-equal)^1)
+local str=Cs(lpegpatterns.unquoted)+C((1-whitespace-equal)^1)
 local setting=Cf(Carg(1)*(whitespace^0*Cg(str*whitespace^0*(equal*whitespace^0*str+Cc(""))))^1,rawset)
 local splitter=setting^1
 function utilities.parsers.options_to_hash(str,target)
   return str and lpegmatch(splitter,str,1,target or {}) or {}
 end
+local splitter=lpeg.tsplitat(" ")
+function utilities.parsers.options_to_array(str)
+  return str and lpegmatch(splitter,str) or {}
+end
 local value=P(lbrace*C((nobrace+nestedbraces)^0)*rbrace)+C(digit^1*lparent*(noparent+nestedparents)^1*rparent)+C((nestedbraces+(1-comma))^1)
 local pattern_a=spaces*Ct(value*(separator*value)^0)
 local function repeater(n,str)
@@ -1745,7 +1843,7 @@ function parsers.keq_to_hash(str)
 end
 local defaultspecification={ separator=",",quote='"' }
 function parsers.csvsplitter(specification)
-  specification=specification and table.setmetatableindex(specification,defaultspecification) or defaultspecification
+  specification=specification and setmetatableindex(specification,defaultspecification) or defaultspecification
   local separator=specification.separator
   local quotechar=specification.quote
   local separator=S(separator~="" and separator or ",")
@@ -1769,7 +1867,7 @@ function parsers.csvsplitter(specification)
   end
 end
 function parsers.rfc4180splitter(specification)
-  specification=specification and table.setmetatableindex(specification,defaultspecification) or defaultspecification
+  specification=specification and setmetatableindex(specification,defaultspecification) or defaultspecification
   local separator=specification.separator 
   local quotechar=P(specification.quote) 
   local dquotechar=quotechar*quotechar  
@@ -1780,7 +1878,7 @@ function parsers.rfc4180splitter(specification)
   local field=escaped+non_escaped+Cc("")
   local record=Ct(field*(separator*field)^1)
   local headerline=record*Cp()
-  local wholeblob=Ct((newline^-1*record)^0)
+  local wholeblob=Ct((newline^(specification.strict and -1 or 1)*record)^0)
   return function(data,getheader)
     if getheader then
       local header,position=lpegmatch(headerline,data)
@@ -1817,20 +1915,20 @@ function parsers.stepper(str,n,action)
     lpegmatch(stepper,str,1,n,action or print)
   end
 end
-local pattern_math=Cs((P("%")/"\\percent "+P("^")*Cc("{")*lpegpatterns.integer*Cc("}")+P(1))^0)
-local pattern_text=Cs((P("%")/"\\percent "+(P("^")/"\\high")*Cc("{")*lpegpatterns.integer*Cc("}")+P(1))^0)
+local pattern_math=Cs((P("%")/"\\percent "+P("^")*Cc("{")*lpegpatterns.integer*Cc("}")+anything)^0)
+local pattern_text=Cs((P("%")/"\\percent "+(P("^")/"\\high")*Cc("{")*lpegpatterns.integer*Cc("}")+anything)^0)
 patterns.unittotex=pattern
 function parsers.unittotex(str,textmode)
   return lpegmatch(textmode and pattern_text or pattern_math,str)
 end
-local pattern=Cs((P("^")/"<sup>"*lpegpatterns.integer*Cc("</sup>")+P(1))^0)
+local pattern=Cs((P("^")/"<sup>"*lpegpatterns.integer*Cc("</sup>")+anything)^0)
 function parsers.unittoxml(str)
   return lpegmatch(pattern,str)
 end
 local cache={}
-local spaces=lpeg.patterns.space^0
+local spaces=lpegpatterns.space^0
 local dummy=function() end
-table.setmetatableindex(cache,function(t,k)
+setmetatableindex(cache,function(t,k)
   local separator=P(k)
   local value=(1-separator)^0
   local pattern=spaces*C(value)*separator^0*Cp()
@@ -1895,6 +1993,18 @@ function utilities.parsers.runtime(time)
   local seconds=mod(time,60)
   return days,hours,minutes,seconds
 end
+local spacing=whitespace^0
+local apply=P("->")
+local method=C((1-apply)^1)
+local token=lbrace*C((1-rbrace)^1)*rbrace+C(anything^1)
+local pattern=spacing*(method*spacing*apply+Carg(1))*spacing*token
+function utilities.parsers.splitmethod(str,default)
+  if str then
+    return lpegmatch(pattern,str,1,default or false)
+  else
+    return default or false,""
+  end
+end
 
 end -- closure
 
@@ -1913,10 +2023,11 @@ local allocate=utilities.storage.allocate
 local setmetatableindex=table.setmetatableindex
 local formatters=string.formatters
 local texget=tex and tex.get or function() return 65536*10*100 end
+local p_stripzeros=lpeg.patterns.stripzeros
 number=number or {}
 local number=number
-number.tonumberf=function(n) return match(format("%.20f",n),"(.-0?)0*$") end 
-number.tonumberg=function(n) return    format("%.20g",n)       end
+number.tonumberf=function(n) return lpegmatch(p_stripzeros,format("%.20f",n)) end
+number.tonumberg=function(n) return format("%.20g",n) end
 local dimenfactors=allocate {
   ["pt"]=1/65536,
   ["in"]=(100/7227)/65536,
@@ -1930,17 +2041,21 @@ local dimenfactors=allocate {
   ["nd"]=(20320/21681)/65536,
   ["nc"]=(5080/65043)/65536
 }
-local function numbertodimen(n,unit,fmt)
+local f_none=formatters["%s%s"]
+local f_true=formatters["%0.5F%s"]
+local function numbertodimen(n,unit,fmt) 
   if type(n)=='string' then
     return n
   else
     unit=unit or 'pt'
+    n=n*dimenfactors[unit]
     if not fmt then
-      fmt="%s%s"
+      fmt=f_none(n,unit)
     elseif fmt==true then
-      fmt="%0.5f%s"
+      fmt=f_true(n,unit)
+    else
+      return formatters[fmt](n,unit)
     end
-    return format(fmt,n*dimenfactors[unit],unit)
   end
 end
 number.maxdimen=1073741823
@@ -2363,15 +2478,6 @@ function statistics.tracefunction(base,tag,...)
     statistics.register(formatters["%s.%s"](tag,name),function() return serialize(stat,"calls") end)
   end
 end
-commands=commands or {}
-function commands.resettimer(name)
-  resettiming(name or "whatever")
-  starttiming(name or "whatever")
-end
-function commands.elapsedtime(name)
-  stoptiming(name or "whatever")
-  context(elapsedtime(name or "whatever"))
-end
 
 end -- closure
 
@@ -2641,7 +2747,7 @@ local sqlescape=lpeg.replacer {
   { "\r\n","\\n" },
   { "\r","\\n" },
 }
-local sqlquoted=lpeg.Cs(lpeg.Cc("'")*sqlescape*lpeg.Cc("'"))
+local sqlquoted=Cs(Cc("'")*sqlescape*Cc("'"))
 lpegpatterns.sqlescape=sqlescape
 lpegpatterns.sqlquoted=sqlquoted
 local luaescape=lpegpatterns.luaescape
@@ -2664,12 +2770,24 @@ local quotedescapers={
 local luaescaper=escapers.lua
 local quotedluaescaper=quotedescapers.lua
 local function replacekeyunquoted(s,t,how,recurse) 
-  local escaper=how and escapers[how] or luaescaper
-  return escaper(replacekey(s,t,how,recurse))
+  if how==false then
+    return replacekey(s,t,how,recurse)
+  else
+    local escaper=how and escapers[how] or luaescaper
+    return escaper(replacekey(s,t,how,recurse))
+  end
 end
 local function replacekeyquoted(s,t,how,recurse) 
-  local escaper=how and quotedescapers[how] or quotedluaescaper
-  return escaper(replacekey(s,t,how,recurse))
+  if how==false then
+    return replacekey(s,t,how,recurse)
+  else
+    local escaper=how and quotedescapers[how] or quotedluaescaper
+    return escaper(replacekey(s,t,how,recurse))
+  end
+end
+local function replaceoptional(l,m,r,t,how,recurse)
+  local v=t[l]
+  return v and v~="" and lpegmatch(replacer,r,1,t,how or "lua",recurse or false) or ""
 end
 local single=P("%") 
 local double=P("%%") 
@@ -2684,11 +2802,16 @@ local nolquoted=lquoted/''
 local norquoted=rquoted/''
 local nolquotedq=lquotedq/''
 local norquotedq=rquotedq/''
-local key=nosingle*((C((1-nosingle )^1)*Carg(1)*Carg(2)*Carg(3))/replacekey    )*nosingle
-local quoted=nolquotedq*((C((1-norquotedq)^1)*Carg(1)*Carg(2)*Carg(3))/replacekeyquoted )*norquotedq
-local unquoted=nolquoted*((C((1-norquoted )^1)*Carg(1)*Carg(2)*Carg(3))/replacekeyunquoted)*norquoted
+local noloptional=P("%?")/''
+local noroptional=P("?%")/''
+local nomoptional=P(":")/''
+local args=Carg(1)*Carg(2)*Carg(3)
+local key=nosingle*((C((1-nosingle  )^1)*args)/replacekey    )*nosingle
+local quoted=nolquotedq*((C((1-norquotedq )^1)*args)/replacekeyquoted )*norquotedq
+local unquoted=nolquoted*((C((1-norquoted )^1)*args)/replacekeyunquoted)*norquoted
+local optional=noloptional*((C((1-nomoptional)^1)*nomoptional*C((1-noroptional)^1)*args)/replaceoptional)*noroptional
 local any=P(1)
-   replacer=Cs((unquoted+quoted+escape+key+any)^0)
+   replacer=Cs((unquoted+quoted+escape+optional+key+any)^0)
 local function replace(str,mapping,how,recurse)
   if mapping and str then
     return lpegmatch(replacer,str,1,mapping,how or "lua",recurse or false) or str
@@ -2798,6 +2921,7 @@ local function change(s,t1,first1,last1,t2,first2,last2)
   end
 end
 function stacker.new(name)
+  local report=logs.reporter("stacker",name or nil)
   local s
   local stack={}
   local list={}
@@ -2838,7 +2962,16 @@ function stacker.new(name)
     end
   end
   local tops={}
-  local top,switch
+  local top=nil
+  local switch=nil
+  local function resolve_reset(mode)
+    if #tops>0 then
+      report("resetting %s left-over states of %a",#tops,name)
+    end
+    tops={}
+    top=nil
+    switch=nil
+  end
   local function resolve_begin(mode)
     if mode then
       switch=mode=="switch"
@@ -2912,8 +3045,7 @@ function stacker.new(name)
     end
   end
   local function resolve_end()
-    local noftop=#top
-    if noftop>0 then
+    if #tops>0 then 
       local result=s.stop(s,top,1,#top)
       remove(tops)
       top=tops[#tops]
@@ -2928,7 +3060,6 @@ function stacker.new(name)
     end
     resolve_end()
   end
-  local report=logs.reporter("stacker",name or nil)
   s={
     name=name or "unknown",
     unset=-1,
@@ -2943,6 +3074,7 @@ function stacker.new(name)
     resolve_begin=resolve_begin,
     resolve_step=resolve_step,
     resolve_end=resolve_end,
+    resolve_reset=resolve_reset,
   }
   return s 
 end
