@@ -2,23 +2,19 @@
 -----------------------------------------------------------------------
 --         FILE:  luaotfload-auxiliary.lua
 --  DESCRIPTION:  part of luaotfload
--- REQUIREMENTS:  luaotfload 2.5
+-- REQUIREMENTS:  luaotfload 2.6
 --       AUTHOR:  Khaled Hosny, Élie Roux, Philipp Gesang
---      VERSION:  2.5
---     MODIFIED:  2014-01-02 21:24:25+0100
 -----------------------------------------------------------------------
 --
 
 --- this file addresses issue #24
 --- https://github.com/lualatex/luaotfload/issues/24#
 
-luaotfload                  = luaotfload or {}
-luaotfload.aux              = luaotfload.aux or { }
-
-local aux                   = luaotfload.aux
+luaotfload                  = luaotfload or { }
 local log                   = luaotfload.log
-local report                = log.report
+local logreport             = log.report
 local fonthashes            = fonts.hashes
+local encodings             = fonts.encodings
 local identifiers           = fonthashes.identifiers
 local fontnames             = fonts.names
 
@@ -36,6 +32,9 @@ local stringbyte            = string.byte
 local stringfind            = string.find
 local tablecopy             = table.copy
 
+local aux                   = { }
+local luaotfload_callbacks  = { }
+
 -----------------------------------------------------------------------
 ---                          font patches
 -----------------------------------------------------------------------
@@ -55,8 +54,8 @@ local start_rewrite_fontname = function ()
       rewrite_fontname,
       "luaotfload.rewrite_fontname")
     rewriting = true
-    report ("log", 1, "aux",
-            "start rewriting tfmdata.name field")
+    logreport ("log", 1, "aux",
+               "start rewriting tfmdata.name field")
   end
 end
 
@@ -64,11 +63,11 @@ aux.start_rewrite_fontname = start_rewrite_fontname
 
 local stop_rewrite_fontname = function ()
   if rewriting == true then
-    luatexbase.remove_fromt_callback
+    luatexbase.remove_from_callback
       ("luaotfload.patch_font", "luaotfload.rewrite_fontname")
     rewriting = false
-    report ("log", 1, "aux",
-            "stop rewriting tfmdata.name field")
+    logreport ("log", 1, "aux",
+               "stop rewriting tfmdata.name field")
   end
 end
 
@@ -90,10 +89,9 @@ local set_sscale_dimens = function (fontdata)
   return fontdata
 end
 
-luatexbase.add_to_callback(
-  "luaotfload.patch_font",
-  set_sscale_dimens,
-  "luaotfload.aux.set_sscale_dimens")
+luaotfload_callbacks [#luaotfload_callbacks + 1] = {
+  "patch_font", set_sscale_dimens, "set_sscale_dimens",
+}
 
 --- fontobj -> int
 local lookup_units = function (fontdata)
@@ -125,10 +123,37 @@ local patch_cambria_domh = function (fontdata)
   end
 end
 
-luatexbase.add_to_callback(
-  "luaotfload.patch_font",
-  patch_cambria_domh,
-  "luaotfload.aux.patch_cambria_domh")
+luaotfload_callbacks [#luaotfload_callbacks + 1] = {
+  "patch_font", patch_cambria_domh, "patch_cambria_domh",
+}
+
+
+--[[doc--
+
+  Add missing field to fonts that lack it. Addresses issue
+  https://github.com/lualatex/luaotfload/issues/253
+
+  This is considered a hack, especially since importing the
+  unicode-math package fixes the problem quite nicely.
+
+--doc]]--
+
+--- fontobj -> unit
+local fixup_fontdata = function (data)
+
+  local t = type (data)
+  --- Some OT fonts like Libertine R lack the resources table, causing
+  --- the fontloader to nil-index.
+  if t == "table" then
+    if data and not data.resources then data.resources = { } end
+  end
+
+end
+
+luaotfload_callbacks [#luaotfload_callbacks + 1] = {
+  "patch_font_unsafe", fixup_fontdata, "fixup_fontdata",
+}
+
 
 --[[doc--
 
@@ -176,17 +201,14 @@ local set_capheight = function (fontdata)
     end
 end
 
-luatexbase.add_to_callback(
-  "luaotfload.patch_font",
-  set_capheight,
-  "luaotfload.aux.set_capheight")
+luaotfload_callbacks [#luaotfload_callbacks + 1] = {
+  "patch_font", set_capheight, "set_capheight",
+}
 
 -----------------------------------------------------------------------
 ---                      glyphs and characters
 -----------------------------------------------------------------------
 
-local agl = fonts.encodings.agl
-
 --- int -> int -> bool
 local font_has_glyph = function (font_id, codepoint)
   local fontdata = fonts.hashes.identifiers[font_id]
@@ -203,7 +225,7 @@ aux.font_has_glyph = font_has_glyph
 local raw_slot_of_name = function (font_id, glyphname)
   local fontdata = font.fonts[font_id]
   if fontdata.type == "virtual" then --- get base font for glyph idx
-    local codepoint  = agl.unicodes[glyphname]
+    local codepoint  = encodings.agl.unicodes[glyphname]
     local glyph      = fontdata.characters[codepoint]
     if fontdata.characters[codepoint] then
       return codepoint
@@ -264,7 +286,7 @@ local indices
 --- int -> (string | false)
 local name_of_slot = function (codepoint)
   if not indices then --- this will load the glyph list
-    local unicodes = agl.unicodes
+    local unicodes = encodings.agl.unicodes
     indices = table.swapped(unicodes)
   end
   local glyphname = indices[codepoint]
@@ -367,7 +389,7 @@ do
 
   local load_chardef = function ()
 
-    report ("both", 1, "aux", "Loading character metadata from %s.", chardef)
+    logreport ("both", 1, "aux", "Loading character metadata from %s.", chardef)
     chardata = dofile (kpse.find_file (chardef, "lua"))
 
     if chardata == nil then
@@ -417,27 +439,27 @@ least one feature.
 local provides_script = function (font_id, asked_script)
   asked_script = stringlower(asked_script)
   if font_id and font_id > 0 then
-    local fontdata = identifiers[font_id].shared.rawdata
-    if fontdata then
-      local fontname = fontdata.metadata.fontname
-      local features = fontdata.resources.features
-      for method, featuredata in next, features do
-        --- where method: "gpos" | "gsub"
-        for feature, data in next, featuredata do
-          if data[asked_script] then
-            report ("log", 1, "aux",
-                    "font no %d (%s) defines feature %s for script %s",
-                    font_id, fontname, feature, asked_script)
-            return true
-          end
+    local tfmdata  = identifiers[font_id] if not tfmdata  then return false end
+    local shared   = tfmdata.shared       if not shared   then return false end
+    local fontdata = shared.rawdata       if not fontdata then return false end
+    local fontname = fontdata.metadata.fontname
+    local features = fontdata.resources.features
+    for method, featuredata in next, features do
+      --- where method: "gpos" | "gsub"
+      for feature, data in next, featuredata do
+        if data[asked_script] then
+          logreport ("log", 1, "aux",
+                     "font no %d (%s) defines feature %s for script %s",
+                     font_id, fontname, feature, asked_script)
+          return true
         end
       end
-      report ("log", 0, "aux",
-              "font no %d (%s) defines no feature for script %s",
-              font_id, fontname, asked_script)
     end
+    logreport ("log", 0, "aux",
+               "font no %d (%s) defines no feature for script %s",
+               font_id, fontname, asked_script)
   end
-  report ("log", 0, "aux", "no font with id %d", font_id)
+  logreport ("log", 0, "aux", "no font with id %d", font_id)
   return false
 end
 
@@ -455,31 +477,31 @@ local provides_language = function (font_id, asked_script, asked_language)
   asked_script     = stringlower(asked_script)
   asked_language   = stringlower(asked_language)
   if font_id and font_id > 0 then
-    local fontdata = identifiers[font_id].shared.rawdata
-    if fontdata then
-      local fontname = fontdata.metadata.fontname
-      local features = fontdata.resources.features
-      for method, featuredata in next, features do
-        --- where method: "gpos" | "gsub"
-        for feature, data in next, featuredata do
-          local scriptdata = data[asked_script]
-          if scriptdata and scriptdata[asked_language] then
-            report ("log", 1, "aux",
-                    "font no %d (%s) defines feature %s "
-                    .. "for script %s with language %s",
-                    font_id, fontname, feature,
-                    asked_script, asked_language)
-            return true
-          end
+    local tfmdata  = identifiers[font_id] if not tfmdata  then return false end
+    local shared   = tfmdata.shared       if not shared   then return false end
+    local fontdata = shared.rawdata       if not fontdata then return false end
+    local fontname = fontdata.metadata.fontname
+    local features = fontdata.resources.features
+    for method, featuredata in next, features do
+      --- where method: "gpos" | "gsub"
+      for feature, data in next, featuredata do
+        local scriptdata = data[asked_script]
+        if scriptdata and scriptdata[asked_language] then
+          logreport ("log", 1, "aux",
+                     "font no %d (%s) defines feature %s "
+                     .. "for script %s with language %s",
+                     font_id, fontname, feature,
+                     asked_script, asked_language)
+          return true
         end
       end
-      report ("log", 0, "aux",
-              "font no %d (%s) defines no feature "
-              .. "for script %s with language %s",
-              font_id, fontname, asked_script, asked_language)
     end
+    logreport ("log", 0, "aux",
+               "font no %d (%s) defines no feature "
+               .. "for script %s with language %s",
+               font_id, fontname, asked_script, asked_language)
   end
-  report ("log", 0, "aux", "no font with id %d", font_id)
+  logreport ("log", 0, "aux", "no font with id %d", font_id)
   return false
 end
 
@@ -527,31 +549,31 @@ local provides_feature = function (font_id,        asked_script,
   asked_feature   = lpegmatch(strip_garbage, asked_feature)
 
   if font_id and font_id > 0 then
-    local fontdata = identifiers[font_id].shared.rawdata
-    if fontdata then
-      local features = fontdata.resources.features
-      local fontname = fontdata.metadata.fontname
-      for method, featuredata in next, features do
-        --- where method: "gpos" | "gsub"
-        local feature = featuredata[asked_feature]
-        if feature then
-          local scriptdata = feature[asked_script]
-          if scriptdata and scriptdata[asked_language] then
-            report ("log", 1, "aux",
-                    "font no %d (%s) defines feature %s "
-                    .. "for script %s with language %s",
-                    font_id, fontname, asked_feature,
-                    asked_script, asked_language)
-            return true
-          end
+    local tfmdata  = identifiers[font_id] if not tfmdata  then return false end
+    local shared   = tfmdata.shared       if not shared   then return false end
+    local fontdata = shared.rawdata       if not fontdata then return false end
+    local features = fontdata.resources.features
+    local fontname = fontdata.metadata.fontname
+    for method, featuredata in next, features do
+      --- where method: "gpos" | "gsub"
+      local feature = featuredata[asked_feature]
+      if feature then
+        local scriptdata = feature[asked_script]
+        if scriptdata and scriptdata[asked_language] then
+          logreport ("log", 1, "aux",
+                     "font no %d (%s) defines feature %s "
+                     .. "for script %s with language %s",
+                     font_id, fontname, asked_feature,
+                     asked_script, asked_language)
+          return true
         end
       end
-      report ("log", 0, "aux",
-              "font no %d (%s) does not define feature %s for script %s with language %s",
-              font_id, fontname, asked_feature, asked_script, asked_language)
     end
+    logreport ("log", 0, "aux",
+               "font no %d (%s) does not define feature %s for script %s with language %s",
+               font_id, fontname, asked_feature, asked_script, asked_language)
   end
-  report ("log", 0, "aux", "no font with id %d", font_id)
+  logreport ("log", 0, "aux", "no font with id %d", font_id)
   return false
 end
 
@@ -817,4 +839,43 @@ end
 
 aux.get_quad = get_quad
 
--- vim:tw=71:sw=2:ts=2:expandtab
+
+-----------------------------------------------------------------------
+---                         initialization
+-----------------------------------------------------------------------
+
+local inject_callbacks = function (lst)
+  if not lst and next (lst) then return false end
+
+  local inject = function (def)
+    local cb, fn, id = unpack (def)
+    cb = tostring (cb)
+    id = tostring (id)
+    if not cb or not fn or not id or not type (fn) == "function" then
+      logreport ("both", 0, "aux", "Invalid callback requested (%q, %s, %q).",
+                 cb, tostring (fn), id)
+      return false
+    end
+    cb = stringformat ("luaotfload.%s",     cb)
+    id = stringformat ("luaotfload.aux.%s", id)
+    logreport ("log", 5, "aux", "Installing callback %q->%q.", cb, id)
+    luatexbase.add_to_callback (cb, fn, id)
+    return true
+  end
+
+  local ret = true
+  for i = 1, #lst do ret = inject (lst [i]) end
+  return ret
+end
+
+return {
+  init = function ()
+    local ret = true
+    luaotfload.aux = aux
+    ret = inject_callbacks (luaotfload_callbacks)
+    return ret
+  end
+}
+
+-- vim:tw=79:sw=2:ts=8:et
+
