@@ -2,38 +2,33 @@
 -------------------------------------------------------------------------------
 --         FILE:  luaotfload-configuration.lua
 --  DESCRIPTION:  config file reader
--- REQUIREMENTS:  Luaotfload 2.5 or above
---       AUTHOR:  Philipp Gesang (Phg), <phg42.2a@gmail.com>
---      VERSION:  same as Luaotfload
---     MODIFIED:  2014-07-13 14:19:32+0200
+-- REQUIREMENTS:  Luaotfload 2.6 or above
+--       AUTHOR:  Philipp Gesang, <phg@phi-gamma.net>
+--       AUTHOR:  Dohyun Kim <nomosnomos@gmail.com>
 -------------------------------------------------------------------------------
 --
 
 if not modules then modules = { } end modules ["luaotfload-configuration"] = {
-  version   = "2.5",
+  version   = "2.6",
   comment   = "part of Luaotfload",
-  author    = "Philipp Gesang",
+  author    = "Philipp Gesang, Dohyun Kim",
   copyright = "Luaotfload Development Team",
   license   = "GNU GPL v2.0"
 }
 
-luaotfload                    = luaotfload or { }
-config                        = config or { }
-config.luaotfload             = { }
-
 local status_file             = "luaotfload-status"
 local luaotfloadstatus        = require (status_file)
 
 local string                  = string
-local stringsub               = string.sub
-local stringexplode           = string.explode
-local stringstrip             = string.strip
 local stringfind              = string.find
+local stringformat            = string.format
+local stringstrip             = string.strip
+local stringsub               = string.sub
 
-local table                   = table
 local tableappend             = table.append
-local tablecopy               = table.copy
 local tableconcat             = table.concat
+local tablecopy               = table.copy
+local table                   = table
 local tabletohash             = table.tohash
 
 local math                    = math
@@ -42,8 +37,10 @@ local mathfloor               = math.floor
 local io                      = io
 local ioloaddata              = io.loaddata
 local iopopen                 = io.popen
+local iowrite                 = io.write
 
 local os                      = os
+local osdate                  = os.date
 local osgetenv                = os.getenv
 
 local lpeg                    = require "lpeg"
@@ -51,9 +48,7 @@ local lpegmatch               = lpeg.match
 local commasplitter           = lpeg.splitat ","
 local equalssplitter          = lpeg.splitat "="
 
-local kpse                    = kpse
 local kpseexpand_path         = kpse.expand_path
-local kpselookup              = kpse.lookup
 
 local lfs                     = lfs
 local lfsisfile               = lfs.isfile
@@ -63,17 +58,13 @@ local file                    = file
 local filejoin                = file.join
 local filereplacesuffix       = file.replacesuffix
 
-
-local parsers                 = luaotfload.parsers
-
-local log                     = luaotfload.log
-local logreport               = log.report
-
-local config_parser           = parsers.config
-local stripslashes            = parsers.stripslashes
-
+local logreport               = print -- overloaded later
 local getwritablepath         = caches.getwritablepath
 
+
+local config_parser -- set later during init
+local stripslashes  -- set later during init
+
 -------------------------------------------------------------------------------
 ---                                SETTINGS
 -------------------------------------------------------------------------------
@@ -98,7 +89,7 @@ local config_paths = {
 }
 
 local valid_formats = tabletohash {
-  "otf",   "ttc", "ttf", "dfont", "afm", "pfb", "pfa",
+  "otf", "ttc", "ttf", "dfont", "afm", "pfb", "pfa",
 }
 
 local feature_presets = {
@@ -127,6 +118,69 @@ local feature_presets = {
   },
 }
 
+--[[doc--
+
+  We allow loading of arbitrary fontloaders. Nevertheless we maintain a
+  list of the “official” ones shipped with Luaotfload so we can emit a
+  different log message.
+
+--doc]]--
+
+local default_fontloader = function ()
+  return luaotfloadstatus and luaotfloadstatus.notes.loader or "reference"
+end
+
+local registered_loaders = {
+  default    = default_fontloader (),
+  reference  = "reference",
+  unpackaged = "unpackaged",
+  context    = "context",
+  tl2014     = "tl2014",
+}
+
+local pick_fontloader = function (s)
+  local ldr = registered_loaders[s]
+  if ldr ~= nil and type (ldr) == "string" then
+    logreport ("log", 2, "conf", "Using predefined fontloader \"%s\".", ldr)
+    return ldr
+  end
+  local idx = stringfind (s, ":")
+  if idx and idx > 2 then
+    if stringsub (s, 1, idx - 1) == "context" then
+      local pth = stringsub (s, idx + 1)
+      pth = stringstrip (pth)
+      logreport ("log", 2, "conf", "Context base path specified at \"%s\".", pth)
+      if lfsisdir (pth) then
+        logreport ("log", 5, "conf", "Context base path exists at \"%s\".", pth)
+        return pth
+      end
+      pth = kpseexpand_path (pth)
+      if lfsisdir (pth) then
+        logreport ("log", 5, "conf", "Context base path exists at \"%s\".", pth)
+        return pth
+      end
+      logreport ("both", 0, "conf", "Context base path not found at \"%s\".", pth)
+    end
+  end
+  return nil
+end
+
+--[[doc--
+
+  The ``post_linebreak_filter`` has been made the default callback for
+  hooking the colorizer into. This helps with the linebreaking whose
+  inserted hyphens would remain unaffected by the coloring otherwise.
+
+  http://tex.stackexchange.com/q/238539/14066
+
+--doc]]--
+
+local permissible_color_callbacks = {
+  default               = "post_linebreak_filter",
+  pre_linebreak_filter  = "pre_linebreak_filter",
+  post_linebreak_filter = "post_linebreak_filter",
+  pre_output_filter     = "pre_output_filter",
+}
 
 
 -------------------------------------------------------------------------------
@@ -147,7 +201,8 @@ local default_config = {
     resolver       = "cached",
     definer        = "patch",
     log_level      = 0,
-    color_callback = "pre_linebreak_filter",
+    color_callback = "post_linebreak_filter",
+    fontloader     = default_fontloader (),
   },
   misc = {
     bisect         = false,
@@ -257,17 +312,20 @@ local set_name_resolver = function ()
     --- replace the resolver from luatex-fonts
     if config.luaotfload.db.resolver == "cached" then
         logreport ("both", 2, "cache", "Caching of name: lookups active.")
-        names.resolvespec  = names.resolve_cached
+        names.resolvespec  = fonts.names.lookup_font_name_cached
     else
-        names.resolvespec  = names.resolve_name
+        names.resolvespec  = fonts.names.lookup_font_name
     end
   end
   return true
 end
 
 local set_loglevel = function ()
-  log.set_loglevel (config.luaotfload.run.log_level)
-  return true
+  if luaotfload then
+    luaotfload.log.set_loglevel (config.luaotfload.run.log_level)
+    return true
+  end
+  return false
 end
 
 local build_cache_paths = function ()
@@ -419,6 +477,7 @@ local option_spec = {
     },
   },
   run = {
+    live = { in_t = boolean_t, }, --- false for the tool, true for TeX run
     resolver = {
       in_t      = string_t,
       out_t     = string_t,
@@ -427,7 +486,29 @@ local option_spec = {
     definer = {
       in_t      = string_t,
       out_t     = string_t,
-      transform = function (d) return d == "generic" and d or "patch" end,
+      transform = function (d)
+        if   d == "generic"      or d == "patch"
+          or d == "info_generic" or d == "info_patch"
+        then
+          return d
+        end
+        return "patch"
+      end,
+    },
+    fontloader = {
+      in_t      = string_t,
+      out_t     = string_t,
+      transform = function (id)
+        local ldr = pick_fontloader (id)
+        if ldr ~= nil then
+          return ldr
+        end
+        logreport ("log", 0, "conf",
+                    "Requested fontloader \"%s\" not defined, "
+                    .. "use at your own risk.",
+                    id)
+        return id
+      end,
     },
     log_level = {
       in_t      = number_t,
@@ -437,9 +518,20 @@ local option_spec = {
     color_callback = {
       in_t      = string_t,
       out_t     = string_t,
-      transform = function (cb)
+      transform = function (cb_spec)
         --- These are the two that make sense.
-        return cb == "pre_output_filter" and cb or "pre_linebreak_filter"
+        local cb = permissible_color_callbacks[cb_spec]
+        if cb then
+          logreport ("log", 3, "conf",
+                     "Using callback \"%s\" for font colorization.",
+                     cb)
+          return cb
+        end
+        logreport ("log", 0, "conf",
+                    "Requested callback identifier \"%s\" invalid, "
+                    .. "falling back to default.",
+                    cb_spec)
+        return permissible_color_callbacks.default
       end,
     },
   },
@@ -474,6 +566,115 @@ local option_spec = {
   },
 }
 
+-------------------------------------------------------------------------------
+---                               FORMATTERS
+-------------------------------------------------------------------------------
+
+local commented = function (str)
+  return ";" .. str
+end
+
+local underscore_replacer = lpeg.replacer ("_", "-", true)
+
+local dashed = function (var)
+  --- INI spec dictates that dashes are valid in variable names, not
+  --- underscores.
+  return underscore_replacer (var) or var
+end
+
+local indent = "  "
+local format_string = function (var, val)
+  return stringformat (indent .. "%s = %s", var, val)
+end
+
+local format_integer = function (var, val)
+  return stringformat (indent .. "%s = %d", var, val)
+end
+
+local format_boolean = function (var, val)
+  return stringformat (indent .. "%s = %s", var, val == true and "true" or "false")
+end
+
+local format_keyval = function (var, val)
+  local list = { }
+  local keys = table.sortedkeys (val)
+  for i = 1, #keys do
+    local key    = keys[i]
+    local subval = val[key]
+    if subval == true then
+      list[#list + 1] = stringformat ("%s", key)
+    else
+      list[#list + 1] = stringformat ("%s=%s", key, val[key])
+    end
+  end
+  if next (list) then
+    return stringformat (indent .. "%s = %s",
+                         var,
+                         tableconcat (list, ","))
+  end
+end
+
+local format_section = function (title)
+  return stringformat ("[%s]", dashed (title))
+end
+
+local conf_header = [==[
+;;-----------------------------------------------------------------------------
+;; Luaotfload Configuration
+;;-----------------------------------------------------------------------------
+;;
+;; This file was generated by luaotfload-tool
+;; on %s. Configuration variables
+;; are documented in the manual to luaotfload.conf(5).
+;;
+;;-----------------------------------------------------------------------------
+
+]==]
+
+local conf_footer = [==[
+
+;; vim:filetype=dosini:expandtab:shiftwidth=2
+]==]
+
+--- Each dumpable variable (the ones mentioned in the man page) receives a
+--- formatter that will be used in dumping the variable. Each value receives a
+--- “commented” flag that indicates whether or not the line should be printed
+--- as a comment.
+
+local formatters = {
+  db = {
+    compress    = { false, format_boolean },
+    formats     = { false, format_string  },
+    max_fonts   = { false, format_integer },
+    scan_local  = { false, format_boolean },
+    skip_read   = { false, format_boolean },
+    strip       = { false, format_boolean },
+    update_live = { false, format_boolean },
+  },
+  default_features = {
+    __default = { true, format_keyval },
+  },
+  misc = {
+    bisect     = { false, format_boolean },
+    statistics = { false, format_boolean },
+    termwidth  = { true , format_integer },
+    version    = { true , format_string  },
+  },
+  paths = {
+    cache_dir    = { false, format_string },
+    names_dir    = { false, format_string },
+    index_file   = { false, format_string },
+    lookups_file = { false, format_string },
+  },
+  run = {
+    color_callback  = { false, format_string  },
+    definer         = { false, format_string  },
+    fontloader      = { false, format_string  },
+    log_level       = { false, format_integer },
+    resolver        = { false, format_string  },
+  },
+}
+
 -------------------------------------------------------------------------------
 ---                           MAIN FUNCTIONALITY
 -------------------------------------------------------------------------------
@@ -666,7 +867,7 @@ local read = function (extra)
     return false
   end
 
-  local parsed = lpegmatch (parsers.config, raw)
+  local parsed = lpegmatch (config_parser, raw)
   if not parsed then
     logreport ("both", 2, "conf", "Error parsing configuration file %q.", readme)
     return false
@@ -689,16 +890,70 @@ local apply_defaults = function ()
   return reconfigure ()
 end
 
+local dump = function ()
+  local sections = table.sortedkeys (config.luaotfload)
+  local confdata = { }
+  for i = 1, #sections do
+    local section    = sections[i]
+    local vars       = config.luaotfload[section]
+    local varnames   = table.sortedkeys (vars)
+    local sformats   = formatters[section]
+    if sformats then
+      confdata[#confdata + 1] = format_section (section)
+      for j = 1, #varnames do
+        local var = varnames[j]
+        local val = vars[var]
+        local comment, sformat = unpack (sformats[var] or { })
+        if not sformat then
+          comment, sformat = unpack (sformats.__default or { })
+        end
+
+        if sformat then
+          local dashedvar = dashed (var)
+          if comment then
+            confdata[#confdata + 1] = commented (sformat (dashedvar, val))
+          else
+            confdata[#confdata + 1] = sformat (dashedvar, val)
+          end
+        end
+
+      end
+      confdata[#confdata + 1] = ""
+    end
+  end
+  if next(confdata) then
+    iowrite (stringformat (conf_header,
+                           osdate ("%Y-%m-d %H:%M:%S", os.time ())))
+    iowrite (tableconcat (confdata, "\n"))
+    iowrite (conf_footer)
+  end
+end
+
 -------------------------------------------------------------------------------
 ---                                 EXPORTS
 -------------------------------------------------------------------------------
 
-luaotfload.default_config = default_config
+return {
+  init = function ()
+    config.luaotfload = { }
+    logreport         = luaotfload.log.report
+    local parsers     = luaotfload.parsers
+    config_parser     = parsers.config
+    stripslashes      = parsers.stripslashes
 
-config.actions = {
-  read             = read,
-  apply            = apply,
-  apply_defaults   = apply_defaults,
-  reconfigure      = reconfigure,
+    luaotfload.default_config = default_config
+    config.actions = {
+      read             = read,
+      apply            = apply,
+      apply_defaults   = apply_defaults,
+      reconfigure      = reconfigure,
+      dump             = dump,
+    }
+    if not apply_defaults () then
+      logreport ("log", 0, "load",
+                 "Configuration unsuccessful: error loading default settings.")
+    end
+    return true
+  end
 }
 
