@@ -1,5 +1,5 @@
 if not modules then modules = { } end modules ["luaotfload-log"] = {
-    version   = "2.5",
+    version   = "2.6",
     comment   = "companion to Luaotfload",
     author    = "Khaled Hosny, Elie Roux, Philipp Gesang",
     copyright = "Luaotfload Development Team",
@@ -91,7 +91,7 @@ local set_logout = function (s, finalizers)
         logout = "redirect"
         local chan = choose_logfile ()
         chan:write (stringformat ("logging initiated at %s",
-                                  osdate ("%Y-%m-%d %h:%m:%s", --- i. e. osdate "%F %T"
+                                  osdate ("%Y-%m-%d %H:%M:%S", --- i. e. osdate "%F %T"
                                           ostime ())))
         local writefile = function (...)
             if select ("#", ...) == 2 then
@@ -118,7 +118,7 @@ local set_logout = function (s, finalizers)
 
         finalizers[#finalizers+1] = function ()
             chan:write (stringformat ("\nlogging finished at %s\n",
-                                      osdate ("%Y-%m-%d %h:%m:%s", --- i. e. osdate "%F %T"
+                                      osdate ("%Y-%m-%d %H:%M:%S", --- i. e. osdate "%F %T"
                                               ostime ())))
             chan:close ()
             texiowrite    = texio.write
@@ -355,52 +355,3 @@ end
 
 texio.reporter = texioreporter
 
---[[doc--
-
-    Adobe Glyph List.
-    -------------------------------------------------------------------
-
-    Context provides a somewhat different font-age.lua from an unclear
-    origin. Unfortunately, the file name it reads from is hard-coded
-    in font-enc.lua, so we have to replace the entire table.
-
-    This shouldn’t cause any complications. Due to its implementation
-    the glyph list will be loaded upon loading a OTF or TTF for the
-    first time during a TeX run. (If one sticks to TFM/OFM then it is
-    never read at all.) For this reason we can install a metatable that
-    looks up the file of our choosing and only falls back to the
-    Context one in case it cannot be found.
-
---doc]]--
-
-if fonts then --- need to be running TeX
-    if next(fonts.encodings.agl) then
-        --- unnecessary because the file shouldn’t be loaded at this time
-        --- but we’re just making sure
-        fonts.encodings.agl = nil
-        collectgarbage"collect"
-    end
-
-
-    fonts.encodings.agl = { }
-
-    setmetatable(fonts.encodings.agl, { __index = function (t, k)
-        if k == "unicodes" then
-            local glyphlist = resolvers.findfile"luaotfload-glyphlist.lua"
-            if glyphlist then
-                report ("log", 1, "load", "loading the Adobe glyph list")
-            else
-                glyphlist = resolvers.findfile"font-age.lua"
-                report ("both", 0, "load",
-                    "loading the extended glyph list from ConTeXt")
-            end
-            local unicodes      = dofile(glyphlist)
-            fonts.encodings.agl = { unicodes = unicodes }
-            return unicodes
-        else
-            return nil
-        end
-    end })
-end
-
--- vim:tw=71:sw=4:ts=4:expandtab
