@@ -1,6 +1,7 @@
 -- luatexko-uhc2utf8.lua
 --
--- Copyright (c) 2013-2015 Dohyun Kim  <nomos at ktug org>
+-- Copyright (c) 2013-2015  Dohyun Kim  <nomos at ktug org>
+--                          Soojin Nam  <jsunam at gmail com>
 --
 -- This work may be distributed and/or modified under the
 -- conditions of the LaTeX Project Public License, either version 1.3c
@@ -12,9 +13,9 @@
 
 luatexbase.provides_module({
   name        = "luatexko-uhc2utf8",
-  version     = 1.8,
-  date        = "2015/04/13",
-  author      = "Dohyun Kim",
+  version     = "1.11",
+  date        = "2015/12/10",
+  author      = "Dohyun Kim, Soojin Nam",
   description = "UHC (CP949) input encoding",
   license     = "LPPL v1.3+",
 })
